@@ -47,6 +47,14 @@ impl BuildableSource<'_> {
             Self::Url(_) => None,
         }
     }
+
+    /// Returns `true` if the source is editable.
+    pub fn is_editable(&self) -> bool {
+        match self {
+            Self::Dist(dist) => dist.is_editable(),
+            Self::Url(url) => url.is_editable(),
+        }
+    }
 }
 
 impl std::fmt::Display for BuildableSource<'_> {
@@ -77,6 +85,14 @@ impl<'a> SourceUrl<'a> {
             Self::Directory(dist) => dist.url,
         }
     }
+
+    /// Returns `true` if the source is editable.
+    pub fn is_editable(&self) -> bool {
+        matches!(
+            self,
+            Self::Directory(DirectorySourceUrl { editable: true, .. })
+        )
+    }
 }
 
 impl std::fmt::Display for SourceUrl<'_> {
@@ -105,9 +121,9 @@ impl std::fmt::Display for DirectSourceUrl<'_> {
 pub struct GitSourceUrl<'a> {
     /// The URL with the revision and subdirectory fragment.
     pub url: &'a VerbatimUrl,
+    pub git: &'a GitUrl,
     /// The URL without the revision and subdirectory fragment.
-    pub git: Cow<'a, GitUrl>,
-    pub subdirectory: Option<Cow<'a, Path>>,
+    pub subdirectory: Option<&'a Path>,
 }
 
 impl std::fmt::Display for GitSourceUrl<'_> {
@@ -120,8 +136,8 @@ impl<'a> From<&'a GitSourceDist> for GitSourceUrl<'a> {
     fn from(dist: &'a GitSourceDist) -> Self {
         Self {
             url: &dist.url,
-            git: Cow::Borrowed(&dist.git),
-            subdirectory: dist.subdirectory.as_deref().map(Cow::Borrowed),
+            git: &dist.git,
+            subdirectory: dist.subdirectory.as_deref(),
         }
     }
 }
@@ -142,7 +158,7 @@ impl<'a> From<&'a PathSourceDist> for PathSourceUrl<'a> {
     fn from(dist: &'a PathSourceDist) -> Self {
         Self {
             url: &dist.url,
-            path: Cow::Borrowed(&dist.path),
+            path: Cow::Borrowed(&dist.install_path),
         }
     }
 }
@@ -151,6 +167,7 @@ impl<'a> From<&'a PathSourceDist> for PathSourceUrl<'a> {
 pub struct DirectorySourceUrl<'a> {
     pub url: &'a Url,
     pub path: Cow<'a, Path>,
+    pub editable: bool,
 }
 
 impl std::fmt::Display for DirectorySourceUrl<'_> {
@@ -163,7 +180,8 @@ impl<'a> From<&'a DirectorySourceDist> for DirectorySourceUrl<'a> {
     fn from(dist: &'a DirectorySourceDist) -> Self {
         Self {
             url: &dist.url,
-            path: Cow::Borrowed(&dist.path),
+            path: Cow::Borrowed(&dist.install_path),
+            editable: dist.editable,
         }
     }
 }
