@@ -30,7 +30,7 @@ use pep440_rs::Version;
 use pep508_rs::PackageName;
 use pypi_types::{Requirement, VerbatimParsedUrl};
 use uv_configuration::{BuildKind, ConfigSettings, SetupPyStrategy};
-use uv_fs::{PythonExt, Simplified};
+use uv_fs::{rename_with_retry, PythonExt, Simplified};
 use uv_toolchain::{Interpreter, PythonEnvironment};
 use uv_types::{BuildContext, BuildIsolation, SourceBuildTrait};
 
@@ -757,7 +757,7 @@ impl SourceBuild {
 
             let from = tmp_dir.path().join(&filename);
             let to = wheel_dir.join(&filename);
-            fs_err::rename(from, to)?;
+            rename_with_retry(from, to).await?;
             Ok(filename)
         } else {
             if self.build_kind != BuildKind::Wheel {
