@@ -569,9 +569,17 @@ pub struct PipCompileArgs {
     pub python_platform: Option<TargetTriple>,
 
     /// Perform a universal resolution, attempting to generate a single `requirements.txt` output
-    /// file that is compatible with all operating systems, architectures and supported Python
-    /// versions.
-    #[arg(long, overrides_with("no_universal"))]
+    /// file that is compatible with all operating systems, architectures, and Python
+    /// implementations.
+    ///
+    /// In universal mode, the current Python version (or user-provided `--python-version`) will be
+    /// treated as a lower bound. For example, `--universal --python-version 3.7` would produce a
+    /// universal resolution for Python 3.7 and later.
+    #[arg(
+        long,
+        overrides_with("no_universal"),
+        conflicts_with("python_platform")
+    )]
     pub universal: bool,
 
     #[arg(long, overrides_with("universal"), hide = true)]
@@ -1404,6 +1412,13 @@ pub struct PipShowArgs {
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 pub struct PipTreeArgs {
+    /// Maximum display depth of the dependency tree
+    #[arg(long, short, default_value_t = 255)]
+    pub depth: u8,
+
+    /// Prune the given package from the display of the dependency tree.
+    #[arg(long)]
+    pub prune: Vec<PackageName>,
     /// Do not de-duplicate repeated dependencies.
     /// Usually, when a package has already displayed its dependencies,
     /// further occurrences will not re-display its dependencies,
@@ -1490,7 +1505,9 @@ pub struct VenvArgs {
     #[arg(long, overrides_with("system"), hide = true)]
     pub no_system: bool,
 
-    /// Install seed packages (`pip`, `setuptools`, and `wheel`) into the virtual environment.
+    /// Install seed packages (one or more of: `pip`, `setuptools`, and `wheel`) into the virtual environment.
+    ///
+    /// Note `setuptools` and `wheel` are not included in Python 3.12+ environments.
     #[arg(long)]
     pub seed: bool,
 
@@ -1538,7 +1555,7 @@ pub struct VenvArgs {
     /// The strategy to use when resolving against multiple index URLs.
     ///
     /// By default, `uv` will stop at the first index on which a given package is available, and
-    /// limit resolutions to those present on that first index (`first-match`. This prevents
+    /// limit resolutions to those present on that first index (`first-match`). This prevents
     /// "dependency confusion" attacks, whereby an attack can upload a malicious package under the
     /// same name to a secondary
     #[arg(long, value_enum, env = "UV_INDEX_STRATEGY")]
@@ -1746,9 +1763,13 @@ pub struct AddArgs {
     pub requirements: Vec<String>,
 
     /// Add the requirements as development dependencies.
-    #[arg(long)]
+    #[arg(long, conflicts_with("optional"))]
     pub dev: bool,
 
+    /// Add the requirements to the specified optional dependency group.
+    #[arg(long, conflicts_with("dev"))]
+    pub optional: Option<ExtraName>,
+
     /// Add the requirements as editables.
     #[arg(long, default_missing_value = "true", num_args(0..=1))]
     pub editable: Option<bool>,
@@ -1812,9 +1833,13 @@ pub struct RemoveArgs {
     pub requirements: Vec<PackageName>,
 
     /// Remove the requirements from development dependencies.
-    #[arg(long)]
+    #[arg(long, conflicts_with("optional"))]
     pub dev: bool,
 
+    /// Remove the requirements from the specified optional dependency group.
+    #[arg(long, conflicts_with("dev"))]
+    pub optional: Option<ExtraName>,
+
     /// Remove the dependency from a specific package in the workspace.
     #[arg(long, conflicts_with = "isolated")]
     pub package: Option<PackageName>,
@@ -1848,12 +1873,23 @@ pub enum ToolCommand {
     Run(ToolRunArgs),
     /// Install a tool
     Install(ToolInstallArgs),
+    /// List installed tools.
+    List(ToolListArgs),
+    /// Uninstall a tool.
+    Uninstall(ToolUninstallArgs),
 }
 
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 pub struct ToolRunArgs {
     /// The command to run.
+    ///
+    /// By default, the package to install is assumed to match the command name.
+    ///
+    /// The name of the command can include an exact version in the format `<package>@<version>`.
+    ///
+    /// If more complex version specification is desired or if the command is provided by a different
+    /// package, use `--from`.
     #[command(subcommand)]
     pub command: ExternalCommand,
 
@@ -1894,13 +1930,13 @@ pub struct ToolRunArgs {
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 pub struct ToolInstallArgs {
-    /// The command to install.
-    pub name: String,
+    /// The package to install commands from.
+    pub package: String,
 
-    /// Use the given package to provide the command.
+    /// The package to install commands from.
     ///
-    /// By default, the package name is assumed to match the command name.
-    #[arg(long)]
+    /// This option is provided for parity with `uv tool run`, but is redundant with `package`.
+    #[arg(long, hide = true)]
     pub from: Option<String>,
 
     /// Include the following extra requirements.
@@ -1937,6 +1973,16 @@ pub struct ToolInstallArgs {
     pub python: Option<String>,
 }
 
+#[derive(Args)]
+#[allow(clippy::struct_excessive_bools)]
+pub struct ToolListArgs;
+
+#[derive(Args)]
+#[allow(clippy::struct_excessive_bools)]
+pub struct ToolUninstallArgs {
+    pub name: String,
+}
+
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 pub struct ToolchainNamespace {
