@@ -1,20 +1,19 @@
+use std::collections::BTreeSet;
 use std::iter::Flatten;
 use std::path::PathBuf;
-use std::{collections::BTreeSet, hash::BuildHasherDefault};
 
 use anyhow::{Context, Result};
 use fs_err as fs;
-use rustc_hash::{FxHashMap, FxHashSet};
+use rustc_hash::{FxBuildHasher, FxHashMap, FxHashSet};
 use url::Url;
 
 use distribution_types::{
-    Diagnostic, InstalledDist, Name, Requirement, UnresolvedRequirement,
-    UnresolvedRequirementSpecification,
+    Diagnostic, InstalledDist, Name, UnresolvedRequirement, UnresolvedRequirementSpecification,
 };
 use pep440_rs::{Version, VersionSpecifiers};
-use pypi_types::VerbatimParsedUrl;
-use uv_interpreter::PythonEnvironment;
+use pypi_types::{Requirement, VerbatimParsedUrl};
 use uv_normalize::PackageName;
+use uv_toolchain::PythonEnvironment;
 use uv_types::InstalledPackagesProvider;
 
 use crate::satisfies::RequirementSatisfaction;
@@ -39,7 +38,7 @@ pub struct SitePackages {
 
 impl SitePackages {
     /// Build an index of installed packages from the given Python executable.
-    pub fn from_executable(venv: &PythonEnvironment) -> Result<SitePackages> {
+    pub fn from_environment(venv: &PythonEnvironment) -> Result<SitePackages> {
         let mut distributions: Vec<Option<InstalledDist>> = Vec::new();
         let mut by_name = FxHashMap::default();
         let mut by_url = FxHashMap::default();
@@ -53,10 +52,9 @@ impl SitePackages {
                         .filter_map(|read_dir| match read_dir {
                             Ok(entry) => match entry.file_type() {
                                 Ok(file_type) => (file_type.is_dir()
-                                    || entry
-                                        .path()
-                                        .extension()
-                                        .map_or(false, |ext| ext == "egg-link"))
+                                    || entry.path().extension().map_or(false, |ext| {
+                                        ext == "egg-link" || ext == "egg-info"
+                                    }))
                                 .then_some(Ok(entry.path())),
                                 Err(err) => Some(Err(err)),
                             },
@@ -259,8 +257,7 @@ impl SitePackages {
         constraints: &[Requirement],
     ) -> Result<SatisfiesResult> {
         let mut stack = Vec::with_capacity(requirements.len());
-        let mut seen =
-            FxHashSet::with_capacity_and_hasher(requirements.len(), BuildHasherDefault::default());
+        let mut seen = FxHashSet::with_capacity_and_hasher(requirements.len(), FxBuildHasher);
 
         // Add the direct requirements to the queue.
         for entry in requirements {
