@@ -24,6 +24,7 @@ use pep508_rs::{MarkerEnvironment, MarkerTree, VerbatimUrl, VerbatimUrlError};
 use platform_tags::{TagCompatibility, TagPriority, Tags};
 use pypi_types::{HashDigest, ParsedArchiveUrl, ParsedGitUrl};
 use uv_configuration::ExtrasSpecification;
+use uv_distribution::VirtualProject;
 use uv_git::{GitReference, GitSha, RepositoryReference, ResolvedRepositoryReference};
 use uv_normalize::{ExtraName, GroupName, PackageName};
 
@@ -317,35 +318,37 @@ impl Lock {
     /// Convert the [`Lock`] to a [`Resolution`] using the given marker environment, tags, and root.
     pub fn to_resolution(
         &self,
-        workspace_root: &Path,
+        project: &VirtualProject,
         marker_env: &MarkerEnvironment,
         tags: &Tags,
-        root_name: &PackageName,
         extras: &ExtrasSpecification,
         dev: &[GroupName],
     ) -> Result<Resolution, LockError> {
         let mut queue: VecDeque<(&Distribution, Option<&ExtraName>)> = VecDeque::new();
+        let mut seen = FxHashSet::default();
 
-        // Add the root distribution to the queue.
-        let root = self
-            .find_by_name(root_name)
-            .expect("found too many distributions matching root")
-            .expect("could not find root");
+        // Add the workspace packages to the queue.
+        for root_name in project.packages() {
+            let root = self
+                .find_by_name(root_name)
+                .expect("found too many distributions matching root")
+                .expect("could not find root");
 
-        // Add the base package.
-        queue.push_back((root, None));
+            // Add the base package.
+            queue.push_back((root, None));
 
-        // Add any extras.
-        match extras {
-            ExtrasSpecification::None => {}
-            ExtrasSpecification::All => {
-                for extra in root.optional_dependencies.keys() {
-                    queue.push_back((root, Some(extra)));
+            // Add any extras.
+            match extras {
+                ExtrasSpecification::None => {}
+                ExtrasSpecification::All => {
+                    for extra in root.optional_dependencies.keys() {
+                        queue.push_back((root, Some(extra)));
+                    }
                 }
-            }
-            ExtrasSpecification::Some(extras) => {
-                for extra in extras {
-                    queue.push_back((root, Some(extra)));
+                ExtrasSpecification::Some(extras) => {
+                    for extra in extras {
+                        queue.push_back((root, Some(extra)));
+                    }
                 }
             }
         }
@@ -362,20 +365,22 @@ impl Lock {
                         }),
                     ))
                 };
-
             for dep in deps {
                 if dep
                     .marker
                     .as_ref()
                     .map_or(true, |marker| marker.evaluate(marker_env, &[]))
                 {
-                    let dep_dist = self.find_by_id(&dep.distribution_id);
-                    let dep_extra = dep.extra.as_ref();
-                    queue.push_back((dep_dist, dep_extra));
+                    if seen.insert((&dep.distribution_id, dep.extra.as_ref())) {
+                        let dep_dist = self.find_by_id(&dep.distribution_id);
+                        let dep_extra = dep.extra.as_ref();
+                        queue.push_back((dep_dist, dep_extra));
+                    }
                 }
             }
             let name = dist.id.name.clone();
-            let resolved_dist = ResolvedDist::Installable(dist.to_dist(workspace_root, tags)?);
+            let resolved_dist =
+                ResolvedDist::Installable(dist.to_dist(project.workspace().root(), tags)?);
             map.insert(name, resolved_dist);
         }
         let diagnostics = vec![];
@@ -404,72 +409,7 @@ impl Lock {
 
         let mut distributions = ArrayOfTables::new();
         for dist in &self.distributions {
-            let mut table = Table::new();
-
-            table.insert("name", value(dist.id.name.to_string()));
-            table.insert("version", value(dist.id.version.to_string()));
-            table.insert("source", value(dist.id.source.to_string()));
-
-            if let Some(ref sdist) = dist.sdist {
-                table.insert("sdist", value(sdist.to_toml()?));
-            }
-
-            if !dist.dependencies.is_empty() {
-                let deps = dist
-                    .dependencies
-                    .iter()
-                    .map(|dep| dep.to_toml(&dist_count_by_name))
-                    .collect::<ArrayOfTables>();
-                table.insert("dependencies", Item::ArrayOfTables(deps));
-            }
-
-            if !dist.optional_dependencies.is_empty() {
-                let mut optional_deps = Table::new();
-                for (extra, deps) in &dist.optional_dependencies {
-                    let deps = deps
-                        .iter()
-                        .map(|dep| dep.to_toml(&dist_count_by_name))
-                        .collect::<ArrayOfTables>();
-                    optional_deps.insert(extra.as_ref(), Item::ArrayOfTables(deps));
-                }
-                table.insert("optional-dependencies", Item::Table(optional_deps));
-            }
-
-            if !dist.dev_dependencies.is_empty() {
-                let mut dev_dependencies = Table::new();
-                for (extra, deps) in &dist.dev_dependencies {
-                    let deps = deps
-                        .iter()
-                        .map(|dep| dep.to_toml(&dist_count_by_name))
-                        .collect::<ArrayOfTables>();
-                    dev_dependencies.insert(extra.as_ref(), Item::ArrayOfTables(deps));
-                }
-                table.insert("dev-dependencies", Item::Table(dev_dependencies));
-            }
-
-            if !dist.wheels.is_empty() {
-                let wheels = dist
-                    .wheels
-                    .iter()
-                    .enumerate()
-                    .map(|(i, wheel)| {
-                        let mut table = wheel.to_toml()?;
-
-                        if dist.wheels.len() > 1 {
-                            // Indent each wheel on a new line.
-                            table.decor_mut().set_prefix("\n\t");
-                            if i == dist.wheels.len() - 1 {
-                                table.decor_mut().set_suffix("\n");
-                            }
-                        }
-
-                        Ok(table)
-                    })
-                    .collect::<anyhow::Result<Array>>()?;
-                table.insert("wheels", value(wheels));
-            }
-
-            distributions.push(table);
+            distributions.push(dist.to_toml(&dist_count_by_name)?);
         }
 
         doc.insert("distribution", Item::ArrayOfTables(distributions));
@@ -800,6 +740,62 @@ impl Distribution {
         .into())
     }
 
+    fn to_toml(&self, dist_count_by_name: &FxHashMap<PackageName, u64>) -> anyhow::Result<Table> {
+        let mut table = Table::new();
+
+        self.id.to_toml(None, &mut table);
+
+        if let Some(ref sdist) = self.sdist {
+            table.insert("sdist", value(sdist.to_toml()?));
+        }
+
+        if !self.dependencies.is_empty() {
+            let deps = each_element_on_its_line_array(
+                self.dependencies
+                    .iter()
+                    .map(|dep| dep.to_toml(dist_count_by_name).into_inline_table()),
+            );
+            table.insert("dependencies", value(deps));
+        }
+
+        if !self.optional_dependencies.is_empty() {
+            let mut optional_deps = Table::new();
+            for (extra, deps) in &self.optional_dependencies {
+                let deps = each_element_on_its_line_array(
+                    deps.iter()
+                        .map(|dep| dep.to_toml(dist_count_by_name).into_inline_table()),
+                );
+                optional_deps.insert(extra.as_ref(), value(deps));
+            }
+            table.insert("optional-dependencies", Item::Table(optional_deps));
+        }
+
+        if !self.dev_dependencies.is_empty() {
+            let mut dev_dependencies = Table::new();
+            for (extra, deps) in &self.dev_dependencies {
+                let deps = each_element_on_its_line_array(
+                    deps.iter()
+                        .map(|dep| dep.to_toml(dist_count_by_name).into_inline_table()),
+                );
+                dev_dependencies.insert(extra.as_ref(), value(deps));
+            }
+            table.insert("dev-dependencies", Item::Table(dev_dependencies));
+        }
+
+        if !self.wheels.is_empty() {
+            let wheels = each_element_on_its_line_array(
+                self.wheels
+                    .iter()
+                    .map(Wheel::to_toml)
+                    .collect::<anyhow::Result<Vec<_>>>()?
+                    .into_iter(),
+            );
+            table.insert("wheels", value(wheels));
+        }
+
+        Ok(table)
+    }
+
     fn find_best_wheel(&self, tags: &Tags) -> Option<usize> {
         let mut best: Option<(TagPriority, usize)> = None;
         for (i, wheel) in self.wheels.iter().enumerate() {
@@ -930,6 +926,21 @@ impl DistributionId {
             source,
         }
     }
+
+    /// Writes this distribution ID inline into the table given.
+    ///
+    /// When a map is given, and if the package name in this ID is unambiguous
+    /// (i.e., it has a count of 1 in the map), then the `version` and `source`
+    /// fields are omitted. In all other cases, including when a map is not
+    /// given, the `version` and `source` fields are written.
+    fn to_toml(&self, dist_count_by_name: Option<&FxHashMap<PackageName, u64>>, table: &mut Table) {
+        let count = dist_count_by_name.and_then(|map| map.get(&self.name).copied());
+        table.insert("name", value(self.name.to_string()));
+        if count.map(|count| count > 1).unwrap_or(true) {
+            table.insert("version", value(self.version.to_string()));
+            self.source.to_toml(table);
+        }
+    }
 }
 
 impl std::fmt::Display for DistributionId {
@@ -994,7 +1005,8 @@ impl From<DistributionId> for DistributionIdForDependency {
 /// variants should be added without changing the relative ordering of other
 /// variants. Otherwise, this could cause the lock file to have a different
 /// canonical ordering of distributions.
-#[derive(Clone, Debug, Eq, Hash, PartialEq, PartialOrd, Ord)]
+#[derive(Clone, Debug, Eq, Hash, PartialEq, PartialOrd, Ord, serde::Deserialize)]
+#[serde(try_from = "SourceWire")]
 enum Source {
     Registry(Url),
     Git(Url, GitSource),
@@ -1131,56 +1143,42 @@ impl Source {
             },
         )
     }
-}
 
-impl std::str::FromStr for Source {
-    type Err = SourceParseError;
-
-    fn from_str(s: &str) -> Result<Source, SourceParseError> {
-        let (kind, url_or_path) = s.split_once('+').ok_or_else(|| SourceParseError::NoPlus {
-            given: s.to_string(),
-        })?;
-        match kind {
-            "registry" => {
-                let url = Url::parse(url_or_path).map_err(|err| SourceParseError::InvalidUrl {
-                    given: s.to_string(),
-                    err,
-                })?;
-                Ok(Source::Registry(url))
+    fn to_toml(&self, table: &mut Table) {
+        let mut source_table = InlineTable::new();
+        match *self {
+            Source::Registry(ref url) => {
+                source_table.insert("registry", Value::from(url.as_str()));
             }
-            "git" => {
-                let mut url =
-                    Url::parse(url_or_path).map_err(|err| SourceParseError::InvalidUrl {
-                        given: s.to_string(),
-                        err,
-                    })?;
-                let git_source = GitSource::from_url(&mut url).map_err(|err| match err {
-                    GitSourceError::InvalidSha => SourceParseError::InvalidSha {
-                        given: s.to_string(),
-                    },
-                    GitSourceError::MissingSha => SourceParseError::MissingSha {
-                        given: s.to_string(),
-                    },
-                })?;
-                Ok(Source::Git(url, git_source))
+            Source::Git(ref url, _) => {
+                source_table.insert("git", Value::from(url.as_str()));
             }
-            "direct" => {
-                let mut url =
-                    Url::parse(url_or_path).map_err(|err| SourceParseError::InvalidUrl {
-                        given: s.to_string(),
-                        err,
-                    })?;
-                let direct_source = DirectSource::from_url(&mut url);
-                Ok(Source::Direct(url, direct_source))
+            Source::Direct(ref url, DirectSource { ref subdirectory }) => {
+                source_table.insert("url", Value::from(url.as_str()));
+                if let Some(ref subdirectory) = *subdirectory {
+                    source_table.insert("subdirectory", Value::from(subdirectory));
+                }
+            }
+            Source::Path(ref path) => {
+                source_table.insert(
+                    "path",
+                    Value::from(serialize_path_with_dot(path).into_owned()),
+                );
+            }
+            Source::Directory(ref path) => {
+                source_table.insert(
+                    "directory",
+                    Value::from(serialize_path_with_dot(path).into_owned()),
+                );
+            }
+            Source::Editable(ref path) => {
+                source_table.insert(
+                    "editable",
+                    Value::from(serialize_path_with_dot(path).into_owned()),
+                );
             }
-            "path" => Ok(Source::Path(PathBuf::from(url_or_path))),
-            "directory" => Ok(Source::Directory(PathBuf::from(url_or_path))),
-            "editable" => Ok(Source::Editable(PathBuf::from(url_or_path))),
-            name => Err(SourceParseError::UnrecognizedSourceName {
-                given: s.to_string(),
-                name: name.to_string(),
-            }),
         }
+        table.insert("source", value(source_table));
     }
 }
 
@@ -1197,16 +1195,6 @@ impl std::fmt::Display for Source {
     }
 }
 
-impl<'de> serde::Deserialize<'de> for Source {
-    fn deserialize<D>(d: D) -> Result<Source, D::Error>
-    where
-        D: serde::de::Deserializer<'de>,
-    {
-        let string = String::deserialize(d)?;
-        string.parse().map_err(serde::de::Error::custom)
-    }
-}
-
 impl Source {
     fn name(&self) -> &str {
         match *self {
@@ -1231,30 +1219,72 @@ impl Source {
     }
 }
 
+#[derive(Clone, Debug, serde::Deserialize)]
+#[serde(untagged)]
+enum SourceWire {
+    Registry {
+        registry: Url,
+    },
+    Git {
+        git: String,
+    },
+    Direct {
+        url: Url,
+        #[serde(default)]
+        subdirectory: Option<String>,
+    },
+    Path {
+        path: PathBuf,
+    },
+    Directory {
+        directory: PathBuf,
+    },
+    Editable {
+        editable: PathBuf,
+    },
+}
+
+impl TryFrom<SourceWire> for Source {
+    type Error = LockError;
+
+    fn try_from(wire: SourceWire) -> Result<Source, LockError> {
+        #[allow(clippy::enum_glob_use)]
+        use self::SourceWire::*;
+
+        match wire {
+            Registry { registry } => Ok(Source::Registry(registry)),
+            Git { git } => {
+                let mut url = Url::parse(&git)
+                    .map_err(|err| SourceParseError::InvalidUrl {
+                        given: git.to_string(),
+                        err,
+                    })
+                    .map_err(LockErrorKind::InvalidGitSourceUrl)?;
+                let git_source = GitSource::from_url(&mut url)
+                    .map_err(|err| match err {
+                        GitSourceError::InvalidSha => SourceParseError::InvalidSha {
+                            given: git.to_string(),
+                        },
+                        GitSourceError::MissingSha => SourceParseError::MissingSha {
+                            given: git.to_string(),
+                        },
+                    })
+                    .map_err(LockErrorKind::InvalidGitSourceUrl)?;
+                Ok(Source::Git(url, git_source))
+            }
+            Direct { url, subdirectory } => Ok(Source::Direct(url, DirectSource { subdirectory })),
+            Path { path } => Ok(Source::Path(path)),
+            Directory { directory } => Ok(Source::Directory(directory)),
+            Editable { editable } => Ok(Source::Editable(editable)),
+        }
+    }
+}
+
 #[derive(Clone, Debug, Eq, Hash, PartialEq, PartialOrd, Ord, serde::Deserialize)]
 struct DirectSource {
     subdirectory: Option<String>,
 }
 
-impl DirectSource {
-    /// Extracts a direct source reference from the query pairs in the given URL.
-    ///
-    /// This also removes the query pairs and hash fragment from the given
-    /// URL in place.
-    fn from_url(url: &mut Url) -> DirectSource {
-        let subdirectory = url.query_pairs().find_map(|(key, val)| {
-            if key == "subdirectory" {
-                Some(val.into_owned())
-            } else {
-                None
-            }
-        });
-        url.set_query(None);
-        url.set_fragment(None);
-        DirectSource { subdirectory }
-    }
-}
-
 /// NOTE: Care should be taken when adding variants to this enum. Namely, new
 /// variants should be added without changing the relative ordering of other
 /// variants. Otherwise, this could cause the lock file to have a different
@@ -1785,16 +1815,9 @@ impl Dependency {
 
     /// Returns the TOML representation of this dependency.
     fn to_toml(&self, dist_count_by_name: &FxHashMap<PackageName, u64>) -> Table {
-        let count = dist_count_by_name
-            .get(&self.distribution_id.name)
-            .copied()
-            .expect("all dependencies have a corresponding distribution");
         let mut table = Table::new();
-        table.insert("name", value(self.distribution_id.name.to_string()));
-        if count > 1 {
-            table.insert("version", value(self.distribution_id.version.to_string()));
-            table.insert("source", value(self.distribution_id.source.to_string()));
-        }
+        self.distribution_id
+            .to_toml(Some(dist_count_by_name), &mut table);
         if let Some(ref extra) = self.extra {
             table.insert("extra", value(extra.to_string()));
         }
@@ -1982,6 +2005,14 @@ enum LockErrorKind {
         #[source]
         ToUrlError,
     ),
+    /// Failed to parse a git source URL.
+    #[error("failed to parse source git URL")]
+    InvalidGitSourceUrl(
+        /// The underlying error that occurred. This includes the
+        /// errant URL in the message.
+        #[source]
+        SourceParseError,
+    ),
     /// An error that occurs when there's an unrecognized dependency.
     ///
     /// That is, a dependency for a distribution that isn't in the lock file.
@@ -2090,20 +2121,6 @@ enum LockErrorKind {
 /// An error that occurs when a source string could not be parsed.
 #[derive(Clone, Debug, thiserror::Error)]
 enum SourceParseError {
-    /// An error that occurs when no '+' could be found.
-    #[error("could not find `+` in source `{given}`")]
-    NoPlus {
-        /// The source string given.
-        given: String,
-    },
-    /// An error that occurs when the source name was unrecognized.
-    #[error("unrecognized name `{name}` in source `{given}`")]
-    UnrecognizedSourceName {
-        /// The source string given.
-        given: String,
-        /// The unrecognized name.
-        name: String,
-    },
     /// An error that occurs when the URL in the source is invalid.
     #[error("invalid URL in source `{given}`")]
     InvalidUrl {
@@ -2139,6 +2156,32 @@ impl std::fmt::Display for HashParseError {
     }
 }
 
+/// Format an array so that each element is on its own line and has a trailing comma.
+///
+/// Example:
+///
+/// ```toml
+/// dependencies = [
+///     { name = "idna" },
+///     { name = "sniffio" },
+/// ]
+/// ```
+fn each_element_on_its_line_array(elements: impl Iterator<Item = InlineTable>) -> Array {
+    let mut array = elements
+        .map(|mut inline_table| {
+            // Each dependency is on its own line and indented.
+            inline_table.decor_mut().set_prefix("\n    ");
+            inline_table
+        })
+        .collect::<Array>();
+    // With a trailing comma, inserting another entry doesn't change the preceding line,
+    // reducing the diff noise.
+    array.set_trailing_comma(true);
+    // The line break between the last element's comma and the closing square bracket.
+    array.set_trailing("\n");
+    array
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -2151,13 +2194,13 @@ version = 1
 [[distribution]]
 name = "a"
 version = "0.1.0"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution]]
 name = "b"
 version = "0.1.0"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution.dependencies]]
@@ -2176,18 +2219,18 @@ version = 1
 [[distribution]]
 name = "a"
 version = "0.1.0"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution]]
 name = "b"
 version = "0.1.0"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution.dependencies]]
 name = "a"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 "#;
         let result: Result<Lock, _> = toml::from_str(data);
         insta::assert_debug_snapshot!(result);
@@ -2201,13 +2244,13 @@ version = 1
 [[distribution]]
 name = "a"
 version = "0.1.0"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution]]
 name = "b"
 version = "0.1.0"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution.dependencies]]
@@ -2225,19 +2268,19 @@ version = 1
 [[distribution]]
 name = "a"
 version = "0.1.0"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution]]
 name = "a"
 version = "0.1.1"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution]]
 name = "b"
 version = "0.1.0"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution.dependencies]]
@@ -2256,24 +2299,24 @@ version = 1
 [[distribution]]
 name = "a"
 version = "0.1.0"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution]]
 name = "a"
 version = "0.1.1"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution]]
 name = "b"
 version = "0.1.0"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution.dependencies]]
 name = "a"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 "#;
         let result: Result<Lock, _> = toml::from_str(data);
         insta::assert_debug_snapshot!(result);
@@ -2287,19 +2330,19 @@ version = 1
 [[distribution]]
 name = "a"
 version = "0.1.0"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution]]
 name = "a"
 version = "0.1.1"
-source = "registry+https://pypi.org/simple"
+source = { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution]]
 name = "b"
 version = "0.1.0"
-source = "registry+https://pypi.org/simple"
+source =  { registry = "https://pypi.org/simple" }
 sdist = { url = "https://example.com", hash = "sha256:37dd54208da7e1cd875388217d5e00ebd4179249f90fb72437e91a35459a0ad3", size = 0 }
 
 [[distribution.dependencies]]
@@ -2317,7 +2360,7 @@ version = 1
 [[distribution]]
 name = "anyio"
 version = "4.3.0"
-source = "registry+https://pypi.org/simple"
+source = { registry = "https://pypi.org/simple" }
 wheels = [{ url = "https://files.pythonhosted.org/packages/14/fd/2f20c40b45e4fb4324834aea24bd4afdf1143390242c0b33774da0e2e34f/anyio-4.3.0-py3-none-any.whl" }]
 "#;
         let result: Result<Lock, _> = toml::from_str(data);
@@ -2332,8 +2375,64 @@ version = 1
 [[distribution]]
 name = "anyio"
 version = "4.3.0"
-source = "path+file:///foo/bar"
+source = { path = "file:///foo/bar" }
 wheels = [{ url = "file:///foo/bar/anyio-4.3.0-py3-none-any.whl", hash = "sha256:048e05d0f6caeed70d731f3db756d35dcc1f35747c8c403364a8332c630441b8" }]
+"#;
+        let result: Result<Lock, _> = toml::from_str(data);
+        insta::assert_debug_snapshot!(result);
+    }
+
+    #[test]
+    fn source_direct_no_subdir() {
+        let data = r#"
+version = 1
+
+[[distribution]]
+name = "anyio"
+version = "4.3.0"
+source = { url = "https://burntsushi.net" }
+"#;
+        let result: Result<Lock, _> = toml::from_str(data);
+        insta::assert_debug_snapshot!(result);
+    }
+
+    #[test]
+    fn source_direct_has_subdir() {
+        let data = r#"
+version = 1
+
+[[distribution]]
+name = "anyio"
+version = "4.3.0"
+source = { url = "https://burntsushi.net", subdirectory = "wat/foo/bar" }
+"#;
+        let result: Result<Lock, _> = toml::from_str(data);
+        insta::assert_debug_snapshot!(result);
+    }
+
+    #[test]
+    fn source_directory() {
+        let data = r#"
+version = 1
+
+[[distribution]]
+name = "anyio"
+version = "4.3.0"
+source = { directory = "path/to/dir" }
+"#;
+        let result: Result<Lock, _> = toml::from_str(data);
+        insta::assert_debug_snapshot!(result);
+    }
+
+    #[test]
+    fn source_editable() {
+        let data = r#"
+version = 1
+
+[[distribution]]
+name = "anyio"
+version = "4.3.0"
+source = { editable = "path/to/dir" }
 "#;
         let result: Result<Lock, _> = toml::from_str(data);
         insta::assert_debug_snapshot!(result);
