@@ -12,8 +12,8 @@ use uv_auth::store_credentials_from_url;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
-    Concurrency, ConfigSettings, ExtrasSpecification, IndexStrategy, NoBinary, NoBuild,
-    PreviewMode, Reinstall, SetupPyStrategy, Upgrade,
+    BuildOptions, Concurrency, ConfigSettings, ExtrasSpecification, IndexStrategy, PreviewMode,
+    Reinstall, SetupPyStrategy, Upgrade,
 };
 use uv_configuration::{KeyringProviderType, TargetTriple};
 use uv_dispatch::BuildDispatch;
@@ -23,9 +23,11 @@ use uv_installer::{SatisfiesResult, SitePackages};
 use uv_requirements::{RequirementsSource, RequirementsSpecification};
 use uv_resolver::{
     DependencyMode, ExcludeNewer, FlatIndex, InMemoryIndex, OptionsBuilder, PreReleaseMode,
-    ResolutionMode,
+    PythonRequirement, ResolutionMode,
+};
+use uv_toolchain::{
+    EnvironmentPreference, Prefix, PythonEnvironment, PythonVersion, Target, ToolchainRequest,
 };
-use uv_toolchain::{Prefix, PythonEnvironment, PythonVersion, SystemPython, Target, Toolchain};
 use uv_types::{BuildIsolation, HashStrategy, InFlight};
 
 use crate::commands::pip::operations::Modifications;
@@ -56,8 +58,7 @@ pub(crate) async fn pip_install(
     connectivity: Connectivity,
     config_settings: &ConfigSettings,
     no_build_isolation: bool,
-    no_build: NoBuild,
-    no_binary: NoBinary,
+    build_options: BuildOptions,
     python_version: Option<PythonVersion>,
     python_platform: Option<TargetTriple>,
     strict: bool,
@@ -92,8 +93,8 @@ pub(crate) async fn pip_install(
         extra_index_urls,
         no_index,
         find_links,
-        no_binary: specified_no_binary,
-        no_build: specified_no_build,
+        no_binary,
+        no_build,
         extras: _,
     } = operations::read_requirements(
         requirements,
@@ -115,17 +116,14 @@ pub(crate) async fn pip_install(
         .collect();
 
     // Detect the current Python interpreter.
-    let system = if system {
-        SystemPython::Required
-    } else {
-        SystemPython::Explicit
-    };
-    let environment = PythonEnvironment::from_toolchain(Toolchain::find(
-        python.as_deref(),
-        system,
-        preview,
+    let environment = PythonEnvironment::find(
+        &python
+            .as_deref()
+            .map(ToolchainRequest::parse)
+            .unwrap_or_default(),
+        EnvironmentPreference::from_system_flag(system, true),
         &cache,
-    )?);
+    )?;
 
     debug!(
         "Using Python {} environment at {}",
@@ -159,13 +157,13 @@ pub(crate) async fn pip_install(
         } else {
             return if let Some(error) = externally_managed.into_error() {
                 Err(anyhow::anyhow!(
-                    "The interpreter at {} is externally managed, and indicates the following:\n\n{}\n\nConsider creating a virtual environment with `uv environment`.",
+                    "The interpreter at {} is externally managed, and indicates the following:\n\n{}\n\nConsider creating a virtual environment with `uv venv`.",
                     environment.root().user_display().cyan(),
                     textwrap::indent(&error, "  ").green(),
                 ))
             } else {
                 Err(anyhow::anyhow!(
-                    "The interpreter at {} is externally managed. Instead, create a virtual environment with `uv environment`.",
+                    "The interpreter at {} is externally managed. Instead, create a virtual environment with `uv venv`.",
                     environment.root().user_display().cyan()
                 ))
             };
@@ -175,7 +173,7 @@ pub(crate) async fn pip_install(
     let _lock = environment.lock()?;
 
     // Determine the set of installed packages.
-    let site_packages = SitePackages::from_executable(&environment)?;
+    let site_packages = SitePackages::from_environment(&environment)?;
 
     // Check if the current environment satisfies the requirements.
     // Ideally, the resolver would be fast enough to let us remove this check. But right now, for large environments,
@@ -220,6 +218,13 @@ pub(crate) async fn pip_install(
 
     let interpreter = environment.interpreter();
 
+    // Determine the Python requirement, if the user requested a specific version.
+    let python_requirement = if let Some(python_version) = python_version.as_ref() {
+        PythonRequirement::from_python_version(interpreter, python_version)
+    } else {
+        PythonRequirement::from_interpreter(interpreter)
+    };
+
     // Determine the environment for the resolution.
     let (tags, markers) = resolution_environment(python_version, python_platform, interpreter)?;
 
@@ -263,11 +268,14 @@ pub(crate) async fn pip_install(
         .platform(interpreter.platform())
         .build();
 
+    // Combine the `--no-binary` and `--no-build` flags from the requirements files.
+    let build_options = build_options.combine(no_binary, no_build);
+
     // Resolve the flat indexes from `--find-links`.
     let flat_index = {
         let client = FlatIndexClient::new(&client, &cache);
         let entries = client.fetch(index_locations.flat_index()).await?;
-        FlatIndex::from_entries(entries, Some(&tags), &hasher, &no_build, &no_binary)
+        FlatIndex::from_entries(entries, Some(&tags), &hasher, &build_options)
     };
 
     // Determine whether to enable build isolation.
@@ -277,10 +285,6 @@ pub(crate) async fn pip_install(
         BuildIsolation::Isolated
     };
 
-    // Combine the `--no-binary` and `--no-build` flags.
-    let no_binary = no_binary.combine(specified_no_binary);
-    let no_build = no_build.combine(specified_no_build);
-
     // Create a shared in-memory index.
     let index = InMemoryIndex::default();
 
@@ -297,16 +301,16 @@ pub(crate) async fn pip_install(
         &index,
         &git,
         &in_flight,
+        index_strategy,
         setup_py,
         config_settings,
         build_isolation,
         link_mode,
-        &no_build,
-        &no_binary,
+        &build_options,
+        exclude_newer,
         concurrency,
         preview,
-    )
-    .with_options(OptionsBuilder::new().exclude_newer(exclude_newer).build());
+    );
 
     let options = OptionsBuilder::new()
         .resolution_mode(resolution_mode)
@@ -330,10 +334,9 @@ pub(crate) async fn pip_install(
         &hasher,
         &reinstall,
         &upgrade,
-        interpreter,
         Some(&tags),
         Some(&markers),
-        None,
+        python_requirement,
         &client,
         &flat_index,
         &index,
@@ -372,16 +375,16 @@ pub(crate) async fn pip_install(
             &index,
             &git,
             &in_flight,
+            index_strategy,
             setup_py,
             config_settings,
             build_isolation,
             link_mode,
-            &no_build,
-            &no_binary,
+            &build_options,
+            exclude_newer,
             concurrency,
             preview,
         )
-        .with_options(OptionsBuilder::new().exclude_newer(exclude_newer).build())
     };
 
     // Sync the environment.
@@ -390,7 +393,7 @@ pub(crate) async fn pip_install(
         site_packages,
         Modifications::Sufficient,
         &reinstall,
-        &no_binary,
+        &build_options,
         link_mode,
         compile,
         &index_locations,
