@@ -8,14 +8,13 @@ use uv_configuration::PreviewMode;
 use uv_fs::Simplified;
 use uv_toolchain::downloads::{self, DownloadResult, PythonDownload, PythonDownloadRequest};
 use uv_toolchain::managed::{InstalledToolchain, InstalledToolchains};
-use uv_toolchain::{requests_from_version_files, ToolchainRequest};
-use uv_warnings::warn_user;
+use uv_toolchain::{requests_from_version_file, ToolchainRequest};
+use uv_warnings::warn_user_once;
 
 use crate::commands::ExitStatus;
 use crate::printer::Printer;
 
 /// Download and install a Python toolchain.
-#[allow(clippy::too_many_arguments)]
 pub(crate) async fn install(
     targets: Vec<String>,
     force: bool,
@@ -26,7 +25,7 @@ pub(crate) async fn install(
     printer: Printer,
 ) -> Result<ExitStatus> {
     if preview.is_disabled() {
-        warn_user!("`uv toolchain install` is experimental and may change without warning.");
+        warn_user_once!("`uv toolchain install` is experimental and may change without warning.");
     }
 
     let start = std::time::Instant::now();
@@ -35,7 +34,7 @@ pub(crate) async fn install(
     let toolchain_dir = toolchains.root();
 
     let requests: Vec<_> = if targets.is_empty() {
-        if let Some(requests) = requests_from_version_files().await? {
+        if let Some(requests) = requests_from_version_file().await? {
             requests
         } else {
             vec![ToolchainRequest::Any]
