@@ -548,6 +548,8 @@ async fn run() -> Result<ExitStatus> {
             let cache = cache.init()?;
 
             commands::pip_tree(
+                args.depth,
+                args.prune,
                 args.no_dedupe,
                 args.shared.strict,
                 args.shared.python.as_deref(),
@@ -719,8 +721,8 @@ async fn run() -> Result<ExitStatus> {
 
             commands::add(
                 args.requirements,
-                args.dev,
                 args.editable,
+                args.dependency_type,
                 args.raw_sources,
                 args.rev,
                 args.tag,
@@ -749,7 +751,7 @@ async fn run() -> Result<ExitStatus> {
 
             commands::remove(
                 args.requirements,
-                args.dev,
+                args.dependency_type,
                 args.package,
                 args.python,
                 globals.toolchain_preference,
@@ -784,7 +786,7 @@ async fn run() -> Result<ExitStatus> {
             // Initialize the cache.
             let cache = cache.init()?.with_refresh(args.refresh);
 
-            commands::run_tool(
+            commands::tool_run(
                 args.command,
                 args.python,
                 args.from,
@@ -812,9 +814,9 @@ async fn run() -> Result<ExitStatus> {
             let cache = cache.init()?.with_refresh(args.refresh);
 
             commands::tool_install(
-                args.name,
-                args.python,
+                args.package,
                 args.from,
+                args.python,
                 args.with,
                 args.force,
                 args.settings,
@@ -828,6 +830,24 @@ async fn run() -> Result<ExitStatus> {
             )
             .await
         }
+        Commands::Tool(ToolNamespace {
+            command: ToolCommand::List(args),
+        }) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let args = settings::ToolListSettings::resolve(args, filesystem);
+            show_settings!(args);
+
+            commands::tool_list(globals.preview, printer).await
+        }
+        Commands::Tool(ToolNamespace {
+            command: ToolCommand::Uninstall(args),
+        }) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let args = settings::ToolUninstallSettings::resolve(args, filesystem);
+            show_settings!(args);
+
+            commands::tool_uninstall(args.name, globals.preview, printer).await
+        }
         Commands::Toolchain(ToolchainNamespace {
             command: ToolchainCommand::List(args),
         }) => {
