@@ -1,33 +1,34 @@
 // The `unreachable_pub` is to silence false positives in RustRover.
 #![allow(dead_code, unreachable_pub)]
 
-use assert_cmd::assert::{Assert, OutputAssertExt};
-use assert_cmd::Command;
-use assert_fs::assert::PathAssert;
-use assert_fs::fixture::{ChildPath, PathChild};
-use predicates::prelude::predicate;
-use regex::Regex;
 use std::borrow::BorrowMut;
 use std::env;
 use std::ffi::OsString;
 use std::iter::Iterator;
 use std::path::{Path, PathBuf};
-use std::process::Output;
+use std::process::{Command, Output};
 use std::str::FromStr;
-use uv_configuration::PreviewMode;
+
+use assert_cmd::assert::{Assert, OutputAssertExt};
+use assert_fs::assert::PathAssert;
+use assert_fs::fixture::{ChildPath, PathChild, PathCreateDir, SymlinkToFile};
+use predicates::prelude::predicate;
+use regex::Regex;
 
 use uv_cache::Cache;
 use uv_fs::Simplified;
 use uv_toolchain::managed::InstalledToolchains;
-use uv_toolchain::{PythonVersion, Toolchain, ToolchainRequest};
+use uv_toolchain::{
+    EnvironmentPreference, PythonVersion, Toolchain, ToolchainPreference, ToolchainRequest,
+};
 
 // Exclude any packages uploaded after this date.
-pub static EXCLUDE_NEWER: &str = "2024-03-25T00:00:00Z";
+static EXCLUDE_NEWER: &str = "2024-03-25T00:00:00Z";
 
 /// Using a find links url allows using `--index-url` instead of `--extra-index-url` in tests
 /// to prevent dependency confusion attacks against our test suite.
 pub const BUILD_VENDOR_LINKS_URL: &str =
-    "https://raw.githubusercontent.com/astral-sh/packse/0.3.27/vendor/links.html";
+    "https://raw.githubusercontent.com/astral-sh/packse/0.3.29/vendor/links.html";
 
 #[doc(hidden)] // Macro and test context only, don't use directly.
 pub const INSTA_FILTERS: &[(&str, &str)] = &[
@@ -42,6 +43,11 @@ pub const INSTA_FILTERS: &[(&str, &str)] = &[
     // Rewrite Windows output to Unix output
     (r"\\([\w\d])", "/$1"),
     (r"uv.exe", "uv"),
+    // uv version display
+    (
+        r"uv(-.*)? \d+\.\d+\.\d+( \(.*\))?",
+        r"uv [VERSION] ([COMMIT] DATE)",
+    ),
     // The exact message is host language dependent
     (
         r"Caused by: .* \(os error 2\)",
@@ -49,7 +55,7 @@ pub const INSTA_FILTERS: &[(&str, &str)] = &[
     ),
 ];
 
-/// Create a context for tests which simplfiies shared behavior across tests.
+/// Create a context for tests which simplifies shared behavior across tests.
 ///
 /// * Set the current directory to a temporary directory (`temp_dir`).
 /// * Set the cache dir to a different temporary directory (`cache_dir`).
@@ -58,6 +64,8 @@ pub const INSTA_FILTERS: &[(&str, &str)] = &[
 pub struct TestContext {
     pub temp_dir: assert_fs::TempDir,
     pub cache_dir: assert_fs::TempDir,
+    pub python_dir: assert_fs::TempDir,
+    pub home_dir: assert_fs::TempDir,
     pub venv: ChildPath,
     pub workspace_root: PathBuf,
 
@@ -82,6 +90,36 @@ impl TestContext {
         new
     }
 
+    /// Add extra standard filtering for messages like "Resolved 10 packages" which
+    /// can differ between platforms.
+    ///
+    /// In some cases, these counts are helpful for the snapshot and should not be filtered.
+    #[must_use]
+    pub fn with_filtered_counts(mut self) -> Self {
+        for verb in &[
+            "Resolved",
+            "Prepared",
+            "Installed",
+            "Uninstalled",
+            "Audited",
+        ] {
+            self.filters.push((
+                format!("{verb} \\d+ packages?"),
+                format!("{verb} [N] packages"),
+            ));
+        }
+        self
+    }
+
+    /// Add extra standard filtering for executable suffixes on the current platform e.g.
+    /// drops `.exe` on Windows.
+    #[must_use]
+    pub fn with_filtered_exe_suffix(mut self) -> Self {
+        self.filters
+            .push((std::env::consts::EXE_SUFFIX.to_string(), String::new()));
+        self
+    }
+
     /// Create a new test context with multiple Python versions.
     ///
     /// Does not create a virtual environment by default, but the first Python version
@@ -89,8 +127,10 @@ impl TestContext {
     ///
     /// See [`TestContext::new`] if only a single version is desired.
     pub fn new_with_versions(python_versions: &[&str]) -> Self {
-        let temp_dir = assert_fs::TempDir::new().expect("Failed to create temp dir");
-        let cache_dir = assert_fs::TempDir::new().expect("Failed to create cache dir");
+        let temp_dir = assert_fs::TempDir::new().expect("Failed to create test working directory");
+        let cache_dir = assert_fs::TempDir::new().expect("Failed to create test cache directory");
+        let python_dir = assert_fs::TempDir::new().expect("Failed to create test Python directory");
+        let home_dir = assert_fs::TempDir::new().expect("Failed to create test home directory");
 
         // Canonicalize the temp dir for consistent snapshot behavior
         let canonical_temp_dir = temp_dir.canonicalize().unwrap();
@@ -122,6 +162,16 @@ impl TestContext {
             )
             .collect();
 
+        // Construct directories for each Python executable on Unix where the executable names
+        // need to be normalized
+        if cfg!(unix) {
+            for (version, executable) in &python_versions {
+                let parent = python_dir.child(version.to_string());
+                parent.create_dir_all().unwrap();
+                parent.child("python3").symlink_to_file(executable).unwrap();
+            }
+        }
+
         let mut filters = Vec::new();
 
         filters.extend(
@@ -146,7 +196,14 @@ impl TestContext {
             filters.extend(
                 Self::path_patterns(executable)
                     .into_iter()
-                    .map(|pattern| (format!("{pattern}python.*"), format!("[PYTHON-{version}]"))),
+                    .map(|pattern| (pattern.to_string(), format!("[PYTHON-{version}]"))),
+            );
+
+            // And for the symlink we created in the test the Python path
+            filters.extend(
+                Self::path_patterns(&python_dir.join(version.to_string()))
+                    .into_iter()
+                    .map(|pattern| (format!("{pattern}.*"), format!("[PYTHON-{version}]"))),
             );
 
             // Add Python patch version filtering unless explicitly requested to ensure
@@ -163,6 +220,16 @@ impl TestContext {
                 .into_iter()
                 .map(|pattern| (pattern, "[TEMP_DIR]/".to_string())),
         );
+        filters.extend(
+            Self::path_patterns(&python_dir)
+                .into_iter()
+                .map(|pattern| (pattern, "[PYTHON_DIR]/".to_string())),
+        );
+        filters.extend(
+            Self::path_patterns(&home_dir)
+                .into_iter()
+                .map(|pattern| (pattern, "[HOME]/".to_string())),
+        );
         filters.extend(
             Self::path_patterns(&workspace_root)
                 .into_iter()
@@ -200,6 +267,8 @@ impl TestContext {
         Self {
             temp_dir,
             cache_dir,
+            python_dir,
+            home_dir,
             venv,
             workspace_root,
             python_version,
@@ -208,249 +277,178 @@ impl TestContext {
         }
     }
 
-    /// Create a `pip compile` command for testing.
-    pub fn compile(&self) -> std::process::Command {
-        let mut command = self.compile_without_exclude_newer();
-        command.arg("--exclude-newer").arg(EXCLUDE_NEWER);
-        command
-    }
-
-    /// Create a `pip compile` command with no `--exclude-newer` option.
+    /// Shared behaviour for almost all test commands.
     ///
-    /// One should avoid using this in tests to the extent possible because
-    /// it can result in tests failing when the index state changes. Therefore,
-    /// if you use this, there should be some other kind of mitigation in place.
-    /// For example, pinning package versions.
-    pub fn compile_without_exclude_newer(&self) -> std::process::Command {
-        let mut cmd = std::process::Command::new(get_bin());
-        cmd.arg("pip")
-            .arg("compile")
+    /// * Use a temporary cache directory
+    /// * Use a temporary virtual environment with the Python version of [`Self`]
+    /// * Don't wrap text output based on the terminal we're in, the test output doesn't get printed
+    ///   but snapshotted to a string.
+    /// * Use a fake `HOME` to avoid accidentally changing the developer's machine.
+    /// * Hide other Python toolchain with `UV_TOOLCHAIN_DIR` and installed interpreters with
+    ///   `UV_TEST_PYTHON_PATH`.
+    /// * Increase the stack size to avoid stack overflows on windows due to large async functions.
+    pub fn add_shared_args(&self, command: &mut Command) {
+        command
             .arg("--cache-dir")
             .arg(self.cache_dir.path())
             .env("VIRTUAL_ENV", self.venv.as_os_str())
             .env("UV_NO_WRAP", "1")
+            .env("HOME", self.home_dir.as_os_str())
             .env("UV_TOOLCHAIN_DIR", "")
             .env("UV_TEST_PYTHON_PATH", &self.python_path())
+            .env("UV_EXCLUDE_NEWER", EXCLUDE_NEWER)
             .current_dir(self.temp_dir.path());
 
         if cfg!(all(windows, debug_assertions)) {
             // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
             // default windows stack of 1MB
-            cmd.env("UV_STACK_SIZE", (8 * 1024 * 1024).to_string());
+            command.env("UV_STACK_SIZE", (8 * 1024 * 1024).to_string());
         }
+    }
 
-        cmd
+    /// Create a `pip compile` command for testing.
+    pub fn pip_compile(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("pip").arg("compile");
+        self.add_shared_args(&mut command);
+        command
+    }
+
+    /// Create a `pip compile` command for testing.
+    pub fn pip_sync(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("pip").arg("sync");
+        self.add_shared_args(&mut command);
+        command
     }
 
     /// Create a `uv venv` command
-    pub fn venv(&self) -> std::process::Command {
-        let mut command = std::process::Command::new(get_bin());
-        command
-            .arg("venv")
-            .arg("--exclude-newer")
-            .arg(EXCLUDE_NEWER)
-            .env("UV_CACHE_DIR", self.cache_dir.path())
-            .env("UV_TOOLCHAIN_DIR", "")
-            .env("UV_TEST_PYTHON_PATH", &self.python_path())
-            .env("UV_NO_WRAP", "1")
-            .env("UV_STACK_SIZE", (2 * 1024 * 1024).to_string())
-            .current_dir(self.temp_dir.as_os_str());
+    pub fn venv(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("venv");
+        self.add_shared_args(&mut command);
+        command.env_remove("VIRTUAL_ENV");
         command
     }
 
     /// Create a `pip install` command with options shared across scenarios.
-    pub fn install(&self) -> std::process::Command {
-        let mut command = self.install_without_exclude_newer();
-        command.arg("--exclude-newer").arg(EXCLUDE_NEWER);
+    pub fn pip_install(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("pip").arg("install");
+        self.add_shared_args(&mut command);
         command
     }
 
-    /// Create a `pip install` command with no `--exclude-newer` option.
-    ///
-    /// One should avoid using this in tests to the extent possible because
-    /// it can result in tests failing when the index state changes. Therefore,
-    /// if you use this, there should be some other kind of mitigation in place.
-    /// For example, pinning package versions.
-    pub fn install_without_exclude_newer(&self) -> std::process::Command {
-        let mut command = std::process::Command::new(get_bin());
-        command
-            .arg("pip")
-            .arg("install")
-            .arg("--cache-dir")
-            .arg(self.cache_dir.path())
-            .env("VIRTUAL_ENV", self.venv.as_os_str())
-            .env("UV_NO_WRAP", "1")
-            .env("UV_TOOLCHAIN_DIR", "")
-            .env("UV_TEST_PYTHON_PATH", &self.python_path())
-            .current_dir(&self.temp_dir);
-
-        if cfg!(all(windows, debug_assertions)) {
-            // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
-            // default windows stack of 1MB
-            command.env("UV_STACK_SIZE", (4 * 1024 * 1024).to_string());
-        }
-
+    /// Create a `pip uninstall` command with options shared across scenarios.
+    pub fn pip_uninstall(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("pip").arg("uninstall");
+        self.add_shared_args(&mut command);
         command
     }
 
     /// Create a `uv sync` command with options shared across scenarios.
-    pub fn sync(&self) -> std::process::Command {
-        let mut command = std::process::Command::new(get_bin());
-        command
-            .arg("sync")
-            .arg("--cache-dir")
-            .arg(self.cache_dir.path())
-            .env("VIRTUAL_ENV", self.venv.as_os_str())
-            .env("UV_TOOLCHAIN_DIR", "")
-            .env("UV_NO_WRAP", "1")
-            .env("UV_TEST_PYTHON_PATH", &self.python_path())
-            .current_dir(&self.temp_dir);
-
-        if cfg!(all(windows, debug_assertions)) {
-            // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
-            // default windows stack of 1MB
-            command.env("UV_STACK_SIZE", (4 * 1024 * 1024).to_string());
-        }
-
+    pub fn sync(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("sync");
+        self.add_shared_args(&mut command);
         command
     }
 
     /// Create a `uv lock` command with options shared across scenarios.
-    pub fn lock(&self) -> std::process::Command {
-        let mut command = self.lock_without_exclude_newer();
-        command
-            .arg("--exclude-newer")
-            .arg(EXCLUDE_NEWER)
-            .env("UV_TOOLCHAIN_DIR", "")
-            .env("UV_TEST_PYTHON_PATH", &self.python_path());
-        command
-    }
-
-    /// Create a `uv lock` command with no `--exclude-newer` option.
-    ///
-    /// One should avoid using this in tests to the extent possible because
-    /// it can result in tests failing when the index state changes. Therefore,
-    /// if you use this, there should be some other kind of mitigation in place.
-    /// For example, pinning package versions.
-    pub fn lock_without_exclude_newer(&self) -> std::process::Command {
-        let mut command = std::process::Command::new(get_bin());
-        command
-            .arg("lock")
-            .arg("--cache-dir")
-            .arg(self.cache_dir.path())
-            .env("VIRTUAL_ENV", self.venv.as_os_str())
-            .env("UV_NO_WRAP", "1")
-            .env("UV_TOOLCHAIN_DIR", "")
-            .env("UV_TEST_PYTHON_PATH", &self.python_path())
-            .current_dir(&self.temp_dir);
-
-        if cfg!(all(windows, debug_assertions)) {
-            // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
-            // default windows stack of 1MB
-            command.env("UV_STACK_SIZE", (4 * 1024 * 1024).to_string());
-        }
-
+    pub fn lock(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("lock");
+        self.add_shared_args(&mut command);
         command
     }
 
     /// Create a `uv toolchain find` command with options shared across scenarios.
-    pub fn toolchain_find(&self) -> std::process::Command {
-        let mut command = std::process::Command::new(get_bin());
+    pub fn toolchain_find(&self) -> Command {
+        let mut command = Command::new(get_bin());
         command
             .arg("toolchain")
             .arg("find")
-            .arg("--cache-dir")
-            .arg(self.cache_dir.path())
-            .env("VIRTUAL_ENV", self.venv.as_os_str())
-            .env("UV_NO_WRAP", "1")
-            .env("UV_TOOLCHAIN_DIR", "")
-            .env("UV_TEST_PYTHON_PATH", &self.python_path())
             .env("UV_PREVIEW", "1")
             .env("UV_TOOLCHAIN_DIR", "")
             .current_dir(&self.temp_dir);
-
-        if cfg!(all(windows, debug_assertions)) {
-            // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
-            // default windows stack of 1MB
-            command.env("UV_STACK_SIZE", (4 * 1024 * 1024).to_string());
-        }
-
+        self.add_shared_args(&mut command);
         command
     }
 
     /// Create a `uv run` command with options shared across scenarios.
-    pub fn run(&self) -> std::process::Command {
-        let mut command = self.run_without_exclude_newer();
+    pub fn run(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("run");
+        self.add_shared_args(&mut command);
+        command
+    }
+
+    /// Create a `uv tool run` command with options shared across scenarios.
+    pub fn tool_run(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("tool").arg("run");
+        self.add_shared_args(&mut command);
+        command
+    }
+
+    /// Create a `uv tool install` command with options shared across scenarios.
+    pub fn tool_install(&self) -> std::process::Command {
+        let mut command = self.tool_install_without_exclude_newer();
         command.arg("--exclude-newer").arg(EXCLUDE_NEWER);
         command
     }
 
-    /// Create a `uv run` command with no `--exclude-newer` option.
+    /// Create a `uv tool install` command with no `--exclude-newer` option.
     ///
     /// One should avoid using this in tests to the extent possible because
     /// it can result in tests failing when the index state changes. Therefore,
     /// if you use this, there should be some other kind of mitigation in place.
     /// For example, pinning package versions.
-    pub fn run_without_exclude_newer(&self) -> std::process::Command {
+    pub fn tool_install_without_exclude_newer(&self) -> std::process::Command {
         let mut command = std::process::Command::new(get_bin());
+        command.arg("tool").arg("install");
+        self.add_shared_args(&mut command);
         command
-            .arg("run")
-            .arg("--cache-dir")
-            .arg(self.cache_dir.path())
-            .env("VIRTUAL_ENV", self.venv.as_os_str())
-            .env("UV_NO_WRAP", "1")
-            .env("UV_TOOLCHAIN_DIR", "")
-            .env("UV_TEST_PYTHON_PATH", &self.python_path())
-            .current_dir(&self.temp_dir);
+    }
 
-        if cfg!(all(windows, debug_assertions)) {
-            // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
-            // default windows stack of 1MB
-            command.env("UV_STACK_SIZE", (4 * 1024 * 1024).to_string());
-        }
+    /// Create a `uv tool list` command with options shared across scenarios.
+    pub fn tool_list(&self) -> std::process::Command {
+        let mut command = std::process::Command::new(get_bin());
+        command.arg("tool").arg("list");
+        self.add_shared_args(&mut command);
+        command
+    }
 
+    /// Create a `uv tool uninstall` command with options shared across scenarios.
+    pub fn tool_uninstall(&self) -> std::process::Command {
+        let mut command = std::process::Command::new(get_bin());
+        command.arg("tool").arg("uninstall");
+        self.add_shared_args(&mut command);
         command
     }
 
     /// Create a `uv add` command for the given requirements.
-    pub fn add(&self, reqs: &[&str]) -> std::process::Command {
-        let mut command = std::process::Command::new(get_bin());
-        command
-            .arg("add")
-            .args(reqs)
-            .arg("--cache-dir")
-            .arg(self.cache_dir.path())
-            .env("VIRTUAL_ENV", self.venv.as_os_str())
-            .env("UV_NO_WRAP", "1")
-            .current_dir(&self.temp_dir);
-
-        if cfg!(all(windows, debug_assertions)) {
-            // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
-            // default windows stack of 1MB
-            command.env("UV_STACK_SIZE", (4 * 1024 * 1024).to_string());
-        }
-
+    pub fn add(&self, reqs: &[&str]) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("add").args(reqs);
+        self.add_shared_args(&mut command);
         command
     }
 
     /// Create a `uv remove` command for the given requirements.
-    pub fn remove(&self, reqs: &[&str]) -> std::process::Command {
-        let mut command = std::process::Command::new(get_bin());
+    pub fn remove(&self, reqs: &[&str]) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("remove").args(reqs);
+        self.add_shared_args(&mut command);
         command
-            .arg("remove")
-            .args(reqs)
-            .arg("--cache-dir")
-            .arg(self.cache_dir.path())
-            .env("VIRTUAL_ENV", self.venv.as_os_str())
-            .env("UV_NO_WRAP", "1")
-            .current_dir(&self.temp_dir);
-
-        if cfg!(all(windows, debug_assertions)) {
-            // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
-            // default windows stack of 1MB
-            command.env("UV_STACK_SIZE", (4 * 1024 * 1024).to_string());
-        }
+    }
 
+    /// Create a `uv clean` command.
+    pub fn clean(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("clean");
+        self.add_shared_args(&mut command);
         command
     }
 
@@ -460,7 +458,7 @@ impl TestContext {
 
     /// Run the given python code and check whether it succeeds.
     pub fn assert_command(&self, command: &str) -> Assert {
-        std::process::Command::new(venv_to_interpreter(&self.venv))
+        Command::new(venv_to_interpreter(&self.venv))
             // Our tests change files in <1s, so we must disable CPython bytecode caching or we'll get stale files
             // https://github.com/python/cpython/issues/75953
             .arg("-B")
@@ -472,7 +470,7 @@ impl TestContext {
 
     /// Run the given python file and check whether it succeeds.
     pub fn assert_file(&self, file: impl AsRef<Path>) -> Assert {
-        std::process::Command::new(venv_to_interpreter(&self.venv))
+        Command::new(venv_to_interpreter(&self.venv))
             // Our tests change files in <1s, so we must disable CPython bytecode caching or we'll get stale files
             // https://github.com/python/cpython/issues/75953
             .arg("-B")
@@ -522,7 +520,23 @@ impl TestContext {
     }
 
     pub fn python_path(&self) -> OsString {
-        std::env::join_paths(self.python_versions.iter().map(|(_, path)| path)).unwrap()
+        if cfg!(unix) {
+            // On Unix, we needed to normalize the Python executable names to `python3` for the tests
+            std::env::join_paths(
+                self.python_versions
+                    .iter()
+                    .map(|(version, _)| self.python_dir.join(version.to_string())),
+            )
+            .unwrap()
+        } else {
+            // On Windows, just join the parent directories of the executables
+            std::env::join_paths(
+                self.python_versions
+                    .iter()
+                    .map(|(_, executable)| executable.parent().unwrap().to_path_buf()),
+            )
+            .unwrap()
+        }
     }
 
     /// Standard snapshot filters _plus_ those for this test context.
@@ -625,7 +639,7 @@ pub fn create_venv_from_executable<P: AsRef<std::path::Path>>(
     cache_dir: &assert_fs::TempDir,
     python: &Path,
 ) {
-    Command::new(get_bin())
+    assert_cmd::Command::new(get_bin())
         .arg("venv")
         .arg(path.as_ref().as_os_str())
         .arg("--cache-dir")
@@ -652,13 +666,14 @@ pub fn python_path_with_versions(
     temp_dir: &assert_fs::TempDir,
     python_versions: &[&str],
 ) -> anyhow::Result<OsString> {
-    Ok(std::env::join_paths(python_toolchains_for_versions(
-        temp_dir,
-        python_versions,
-    )?)?)
+    Ok(std::env::join_paths(
+        python_toolchains_for_versions(temp_dir, python_versions)?
+            .into_iter()
+            .map(|path| path.parent().unwrap().to_path_buf()),
+    )?)
 }
 
-/// Create a `PATH` with the requested Python versions available in order.
+/// Returns a list of Python executables for the given versions.
 ///
 /// Generally this should be used with `UV_TEST_PYTHON_PATH`.
 pub fn python_toolchains_for_versions(
@@ -668,46 +683,16 @@ pub fn python_toolchains_for_versions(
     let cache = Cache::from_path(temp_dir.child("cache").to_path_buf()).init()?;
     let selected_pythons = python_versions
         .iter()
-        .flat_map(|python_version| {
-            let inner = InstalledToolchains::from_settings()
-                .map(|toolchains| {
-                    toolchains
-                        .find_version(
-                            &PythonVersion::from_str(python_version)
-                                .expect("Tests should use a valid Python version"),
-                        )
-                        .expect("Tests are run on a supported platform")
-                        .map(|toolchain| {
-                            toolchain
-                                .executable()
-                                .parent()
-                                .expect("Executables must exist in a directory")
-                                .to_path_buf()
-                        })
-                        .collect::<Vec<_>>()
-                })
-                .unwrap_or_default();
-            if inner.is_empty() {
-                // Fallback to a system lookup if we failed to find one in the toolchain directory
-                if let Ok(toolchain) = Toolchain::find(
-                    Some(ToolchainRequest::parse(python_version)),
-                    // Without required, we could pick the current venv here and the test fails
-                    // because the venv subcommand requires a system interpreter.
-                    uv_toolchain::SystemPython::Required,
-                    PreviewMode::Enabled,
-                    &cache,
-                ) {
-                    vec![toolchain
-                        .into_interpreter()
-                        .sys_executable()
-                        .parent()
-                        .expect("Python executable should always be in a directory")
-                        .to_path_buf()]
-                } else {
-                    panic!("Could not find Python {python_version} for test");
-                }
+        .map(|python_version| {
+            if let Ok(toolchain) = Toolchain::find(
+                &ToolchainRequest::parse(python_version),
+                EnvironmentPreference::OnlySystem,
+                ToolchainPreference::PreferInstalledManaged,
+                &cache,
+            ) {
+                toolchain.into_interpreter().sys_executable().to_owned()
             } else {
-                inner
+                panic!("Could not find Python {python_version} for test");
             }
         })
         .collect::<Vec<_>>();
@@ -730,7 +715,7 @@ pub enum WindowsFilters {
 ///
 /// This function is derived from `insta_cmd`s `spawn_with_info`.
 pub fn run_and_format<T: AsRef<str>>(
-    mut command: impl BorrowMut<std::process::Command>,
+    mut command: impl BorrowMut<Command>,
     filters: impl AsRef<[(T, T)]>,
     function_name: &str,
     windows_filters: Option<WindowsFilters>,
@@ -800,10 +785,10 @@ pub fn run_and_format<T: AsRef<str>>(
                 for i in 1..20 {
                     for verb in match windows_filters {
                         WindowsFilters::Platform => {
-                            ["Resolved", "Downloaded", "Installed", "Uninstalled"].iter()
+                            ["Resolved", "Prepared", "Installed", "Uninstalled"].iter()
                         }
                         WindowsFilters::Universal => {
-                            ["Downloaded", "Installed", "Uninstalled"].iter()
+                            ["Prepared", "Installed", "Uninstalled"].iter()
                         }
                     } {
                         snapshot = snapshot.replace(
