@@ -11,7 +11,7 @@ mod common;
 /// Create a `pip compile` command, overwriting defaults for any settings that vary based on machine
 /// and operating system.
 fn command(context: &TestContext) -> Command {
-    let mut command = context.compile();
+    let mut command = context.pip_compile();
     command
         .env("UV_LINK_MODE", "clone")
         .env("UV_CONCURRENT_DOWNLOADS", "50")
@@ -57,6 +57,7 @@ fn resolve_uv_toml() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -128,6 +129,7 @@ fn resolve_uv_toml() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -138,6 +140,7 @@ fn resolve_uv_toml() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -146,6 +149,7 @@ fn resolve_uv_toml() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -183,6 +187,7 @@ fn resolve_uv_toml() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -254,6 +259,7 @@ fn resolve_uv_toml() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -264,6 +270,7 @@ fn resolve_uv_toml() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -272,6 +279,7 @@ fn resolve_uv_toml() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -310,6 +318,7 @@ fn resolve_uv_toml() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -381,6 +390,7 @@ fn resolve_uv_toml() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -391,6 +401,7 @@ fn resolve_uv_toml() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -399,6 +410,7 @@ fn resolve_uv_toml() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -469,6 +481,7 @@ fn resolve_pyproject_toml() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -540,6 +553,7 @@ fn resolve_pyproject_toml() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -550,6 +564,7 @@ fn resolve_pyproject_toml() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -558,6 +573,7 @@ fn resolve_pyproject_toml() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -597,6 +613,7 @@ fn resolve_pyproject_toml() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -645,6 +662,7 @@ fn resolve_pyproject_toml() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -655,6 +673,7 @@ fn resolve_pyproject_toml() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -663,6 +682,7 @@ fn resolve_pyproject_toml() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -711,6 +731,7 @@ fn resolve_pyproject_toml() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -782,6 +803,7 @@ fn resolve_pyproject_toml() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -792,6 +814,7 @@ fn resolve_pyproject_toml() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -800,6 +823,7 @@ fn resolve_pyproject_toml() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -862,6 +886,7 @@ fn resolve_index_url() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -956,6 +981,7 @@ fn resolve_index_url() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -966,6 +992,7 @@ fn resolve_index_url() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -974,6 +1001,7 @@ fn resolve_index_url() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -1013,6 +1041,7 @@ fn resolve_index_url() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -1129,6 +1158,7 @@ fn resolve_index_url() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -1139,6 +1169,7 @@ fn resolve_index_url() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -1147,6 +1178,7 @@ fn resolve_index_url() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -1209,6 +1241,7 @@ fn resolve_find_links() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -1275,6 +1308,7 @@ fn resolve_find_links() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -1285,6 +1319,7 @@ fn resolve_find_links() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -1293,6 +1328,7 @@ fn resolve_find_links() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -1354,6 +1390,7 @@ fn resolve_top_level() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -1402,6 +1439,7 @@ fn resolve_top_level() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -1412,6 +1450,7 @@ fn resolve_top_level() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -1420,6 +1459,7 @@ fn resolve_top_level() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -1474,6 +1514,7 @@ fn resolve_top_level() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -1567,6 +1608,7 @@ fn resolve_top_level() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -1577,6 +1619,7 @@ fn resolve_top_level() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -1585,6 +1628,7 @@ fn resolve_top_level() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -1622,6 +1666,7 @@ fn resolve_top_level() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -1715,6 +1760,7 @@ fn resolve_top_level() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -1725,6 +1771,7 @@ fn resolve_top_level() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -1733,6 +1780,7 @@ fn resolve_top_level() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -1794,6 +1842,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -1842,6 +1891,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -1852,6 +1902,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -1860,6 +1911,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -1904,6 +1956,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -1952,6 +2005,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -1962,6 +2016,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -1970,6 +2025,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -2014,6 +2070,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -2062,6 +2119,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -2072,6 +2130,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -2080,6 +2139,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
@@ -2126,6 +2186,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
         isolated: false,
         show_settings: true,
         preview: Disabled,
+        toolchain_preference: OnlySystem,
     }
     CacheSettings {
         no_cache: false,
@@ -2174,6 +2235,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
             prerelease: IfNecessaryOrExplicit,
             output_file: None,
             no_strip_extras: false,
+            no_strip_markers: false,
             no_annotate: false,
             no_header: false,
             custom_compile_command: None,
@@ -2184,6 +2246,7 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
             ),
             python_version: None,
             python_platform: None,
+            universal: false,
             exclude_newer: Some(
                 ExcludeNewer(
                     2024-03-25T00:00:00Z,
@@ -2192,6 +2255,148 @@ fn resolve_user_configuration() -> anyhow::Result<()> {
             no_emit_package: [],
             emit_index_url: false,
             emit_find_links: false,
+            emit_build_options: false,
+            emit_marker_expression: false,
+            emit_index_annotation: false,
+            annotation_style: Split,
+            link_mode: Clone,
+            compile_bytecode: false,
+            require_hashes: false,
+            upgrade: None,
+            reinstall: None,
+            concurrency: Concurrency {
+                downloads: 50,
+                builds: 16,
+                installs: 8,
+            },
+        },
+    }
+
+    ----- stderr -----
+    "###
+    );
+
+    Ok(())
+}
+
+/// Read from a `pyproject.toml` file in the current directory. In this case, the `pyproject.toml`
+/// file uses the Poetry schema.
+#[test]
+#[cfg_attr(
+    windows,
+    ignore = "Configuration tests are not yet supported on Windows"
+)]
+fn resolve_poetry_toml() -> anyhow::Result<()> {
+    let context = TestContext::new("3.12");
+
+    // Write a `uv.toml` file to the directory.
+    let config = context.temp_dir.child("pyproject.toml");
+    config.write_str(indoc::indoc! {r#"
+        [tool.poetry]
+        name = "project"
+        version = "0.1.0"
+
+        [tool.poetry.dependencies]
+        python = "^3.10"
+        rich = "^13.7.1"
+
+        [build-system]
+        requires = ["poetry-core"]
+        build-backend = "poetry.core.masonry.api"
+
+        [tool.uv.pip]
+        resolution = "lowest-direct"
+    "#})?;
+
+    let requirements_in = context.temp_dir.child("requirements.in");
+    requirements_in.write_str("anyio>3.0.0")?;
+
+    // Resolution should use the lowest direct version, and generate hashes.
+    uv_snapshot!(context.filters(), command(&context)
+        .arg("--show-settings")
+        .arg("requirements.in"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    GlobalSettings {
+        quiet: false,
+        verbose: 0,
+        color: Auto,
+        native_tls: false,
+        connectivity: Online,
+        isolated: false,
+        show_settings: true,
+        preview: Disabled,
+        toolchain_preference: OnlySystem,
+    }
+    CacheSettings {
+        no_cache: false,
+        cache_dir: Some(
+            "[CACHE_DIR]/",
+        ),
+    }
+    PipCompileSettings {
+        src_file: [
+            "requirements.in",
+        ],
+        constraint: [],
+        override: [],
+        overrides_from_workspace: [],
+        refresh: None(
+            Timestamp(
+                SystemTime {
+                    tv_sec: [TIME],
+                    tv_nsec: [TIME],
+                },
+            ),
+        ),
+        settings: PipSettings {
+            index_locations: IndexLocations {
+                index: None,
+                extra_index: [],
+                flat_index: [],
+                no_index: false,
+            },
+            python: None,
+            system: false,
+            extras: None,
+            break_system_packages: false,
+            target: None,
+            prefix: None,
+            index_strategy: FirstIndex,
+            keyring_provider: Disabled,
+            no_build_isolation: false,
+            build_options: BuildOptions {
+                no_binary: None,
+                no_build: None,
+            },
+            strict: false,
+            dependency_mode: Transitive,
+            resolution: LowestDirect,
+            prerelease: IfNecessaryOrExplicit,
+            output_file: None,
+            no_strip_extras: false,
+            no_strip_markers: false,
+            no_annotate: false,
+            no_header: false,
+            custom_compile_command: None,
+            generate_hashes: false,
+            setup_py: Pep517,
+            config_setting: ConfigSettings(
+                {},
+            ),
+            python_version: None,
+            python_platform: None,
+            universal: false,
+            exclude_newer: Some(
+                ExcludeNewer(
+                    2024-03-25T00:00:00Z,
+                ),
+            ),
+            no_emit_package: [],
+            emit_index_url: false,
+            emit_find_links: false,
+            emit_build_options: false,
             emit_marker_expression: false,
             emit_index_annotation: false,
             annotation_style: Split,
